import unittest
import datetime
import shutil
import logging
logging.basicConfig(level=logging.INFO)

import wed


class DatabaseTest(unittest.TestCase):
    def test_basic(self):
        db = wed.Database()
        # db.local_cache_path = "test_cache"

        dataset = db.open_dataset(wed.DataType.WIND)
        t = datetime.datetime(2017, 5, 12, 20, 4)
        print(dataset.read(t, wed.Coordinate(0, 0)))
        print(dataset.read(t, wed.Coordinate(12, 150)))

        # shutil.rmtree(db.local_cache_path)

    def test_negative(self):
        db = wed.Database()
        dataset = db.open_dataset(wed.DataType.WIND)
        t = datetime.datetime(2017, 5, 12, 20, 4)

        # make sure that values at -90 is the same as 270
        v1 = dataset.read(t, wed.Coordinate(0, -90))
        v2 = dataset.read(t, wed.Coordinate(0, 270))
        self.assertEqual(v1, v2)

    def test_range(self):
        db = wed.Database()
        dataset = db.open_dataset(wed.DataType.WIND)
        t = datetime.datetime(2019, 5, 12, 20, 4)
        with self.assertRaises(Exception):
            dataset.read(t, wed.Coordinate(0, -90))

    def test_basic_wave(self):
        db = wed.Database()
        # db.local_cache_path = "test_cache"

        dataset = db.open_dataset(wed.DataType.WAVE)
        t = datetime.datetime(2017, 5, 12, 20, 4)
        print(dataset.read(t, wed.Coordinate(0, 0)))
        print(dataset.read(t, wed.Coordinate(12, 150)))



import os
import datetime
import functools
import logging

import cfgrib
import numpy as np

from wed import settings
from wed.path import DatasetPath, DatafilePath
from wed.utils import Coordinate, DataType
from wed.network import download


class Database:
    def __init__(self):
        self.bucket = settings.DEFAULT_BUCKET
        self.original_services = settings.DEFAULT_SERVICES
        self.local_cache_path = settings.DEFAULT_CACHE_PATH

    def open_dataset(self, type: DataType):
        service = self.original_services[type]
        path = DatasetPath(
            self.bucket, self.local_cache_path, type, service)
        return Dataset(path)


# Use the LRU cache property to avoid reopenning the dataset each time
@functools.lru_cache(maxsize=10)
def get_dataset(path: str):
    return cfgrib.open_dataset(path)

class Dataset:
    def __init__(self, path: DatasetPath):
        self.path = path

    def read(self, time: datetime.datetime, coordinate: Coordinate):
        # Make sure we are in the time range
        time_range = settings.get_time_range(self.path.type, self.path.service)
        if time_range['max'] < time or time_range['min'] > time:
            raise Exception("Time out of bounds")

        fp = DatafilePath(self.path, time)
        if not os.path.exists(fp.local_path):
            logging.info("Datafile at " + fp.local_path + " not found. Downloading...")
            download(fp)
            logging.info("Download successful")

        dataset = get_dataset(fp.local_path)  # get from cache or create

        # get the coordinates considering the latitude range
        longitude = coordinate.longitude
        while longitude < dataset.longitude[0].data:
            longitude += 360
        while longitude > dataset.longitude[-1].data:
            longitude -= 360

        # read the right value by interpolating it
        value = dataset.interp({
            'latitude': coordinate.latitude,
            'longitude': longitude,
            'time': np.datetime64(time)
        }, method='linear')

        # cast the xarray dataset to a dict
        retval = {}
        for var in value.data_vars:
            retval[var] = float(value[var].values)
        return retval



from enum import Enum, auto


class DataType(Enum):
    WIND = auto()
    WAVE = auto()
    CURRENT = auto()

    def __str__(self):
        return self.name.lower()

class Service(Enum):
    ECMWF = auto()

    def __str__(self):
        return self.name.lower()

class Coordinate:
    def __init__(self, latitude: float, longitude: float):
        self.latitude = latitude
        self.longitude = longitude



import os
import json
import datetime
from wed.utils import DataType, Service


## List the remote path to the datasets.
# You Need to update it when adding new stuff
settings_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "settings.json")
with open(settings_path) as settings_file:
    SETTINGS = json.load(settings_file)

def get_prefix(type: DataType, service: Service):
    return SETTINGS[str(type)][str(service)]['prefix']

def get_time_range(type: DataType, service: Service):
    range = SETTINGS[str(type)][str(service)]['range']['date']
    return {
        'min': datetime.datetime.strptime(range["min"], '%Y-%m-%d'),
        'max': datetime.datetime.strptime(range["max"], '%Y-%m-%d')
    }

# Default values
DEFAULT_BUCKET = "dice-weather-data"
DEFAULT_CACHE_PATH = os.environ['HOME'] + "/.wed-cache"
DEFAULT_SERVICES = {
    DataType.WIND: Service.ECMWF,
    DataType.WAVE: Service.ECMWF
}
